/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.io.*;
import java.net.*;
import java.util.*;
import java.nio.*;
import java.nio.charset.*;
import java.nio.channels.*;

public class ChainChannel implements IChainChannel {
	final String MESSAGE_DELIMITER = "\n";
	String myAddress = "unknown";
	Vector<ByteBuffer> myOutputBufferList = new Vector<ByteBuffer>();
	String myInputBuffer = new String();
	IChainChannelListener myListener = null;
	SocketChannel myChannel = null;
	SelectionKey myKey = null;
	CharsetDecoder myDecoder = Charset.forName("ISO-8859-1").newDecoder();

	public ChainChannel(Selector selector, SocketChannel channel, IChainChannelListener listener) {
		myListener = listener;
		myChannel = channel;
		try {
			myKey = myChannel.register(selector, SelectionKey.OP_READ | SelectionKey.OP_WRITE );
		} catch(Exception e) {
			e.printStackTrace();
		}
		myKey.attach(this);
		if (myKey != null) onConnected();
	}

	public String getAddress() {
		return myAddress;
	}

	public boolean send(ChainMessage msg) {
		try {
			return send(msg.toString());
		} catch(Exception e){
			e.printStackTrace();
		}
		return false;
	}
	
	private synchronized boolean send(String data) {
		//System.out.println("S>" + data);
		try {
			if (myKey == null) return false;
			if (myKey.isValid()) {
				ByteBuffer tmpBuffer = ByteBuffer.wrap((data + MESSAGE_DELIMITER).getBytes());
				myOutputBufferList.add(tmpBuffer);
				myKey.interestOps(myKey.interestOps() | SelectionKey.OP_WRITE);
				myKey.selector().wakeup();
				return true;
			} else {
				return false;
			}
		} catch(Exception e){
			e.printStackTrace();
		}
		return false;
	}
	
	private boolean processProtocol(String data) {
		if (myListener == null) return false;
		myListener.onReceived(new ChainMessage(this, data));
		return true;
	}
	
	private void onClosed() {
		if (myListener != null) myListener.onClosed(new ChainMessage(this));
	}
	
	private void onConnected() {
		try { myAddress = myChannel.socket().getLocalAddress().getHostName(); } catch (Exception e) {}
		if (myListener != null) myListener.onConnected(new ChainMessage(this));
	}

	private boolean process(String data) {
		boolean ret = true;
		myInputBuffer = myInputBuffer.concat(data);
		String[] tmpCmdList = myInputBuffer.split(MESSAGE_DELIMITER);
		int tmpCmdListSize = tmpCmdList.length;
		if (myInputBuffer.endsWith(MESSAGE_DELIMITER)) {
			myInputBuffer = new String();
		} else {
			tmpCmdListSize--;
			myInputBuffer = tmpCmdList[tmpCmdListSize];
		}
		for (int i = 0; i < tmpCmdListSize; i++) {
			if (processProtocol(tmpCmdList[i]) == false) ret = false;
		}
		return ret;
	}
	
	private boolean process(ByteBuffer data) {
		try {
			if (data == null) return false;
			int tmpLast = data.position();
			if (tmpLast == 0) return false;
			data.position(0);
			data.limit(tmpLast);
			return process(myDecoder.decode(data).toString());
		} catch(Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	public boolean process() {
		if (myChannel == null || myChannel == null || myKey == null) {
			return false;
		} else {
			if (myKey.isValid()) {
				if (myKey.isReadable()) {
					ByteBuffer tmpBuffer = ByteBuffer.allocate(64);
					try {
						if (myChannel.read(tmpBuffer) == -1) {
							myChannel.close();
							myKey.cancel();
						}
					} catch(Exception e) {
						//e.printStackTrace();
						try {
							myChannel.close();
							myKey.cancel();
						} catch(Exception ex) {
							e.printStackTrace();
						}
					}
					process(tmpBuffer);
				}
			}
			if (myKey.isValid()) {
				if (myKey.isWritable()) {
					try {
						ByteBuffer tmpBuffer = null;
						if (myOutputBufferList.size() > 0) tmpBuffer = myOutputBufferList.remove(0);
						if (tmpBuffer != null) myChannel.write(tmpBuffer);
						if (myOutputBufferList.size() == 0) myKey.interestOps(myKey.interestOps() & ~SelectionKey.OP_WRITE);
					} catch(Exception e) {
						try {
							myChannel.close();
							myKey.cancel();
						} catch(Exception ex) {
							e.printStackTrace();
						}
					}
				}
			}
			if (myKey.isValid() == false) {
				myKey = null;
				onClosed();
			}
		}
		return true;		
	}
	
	
}