/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.io.*;
import java.net.*;
import java.nio.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.nio.charset.*;
import java.nio.channels.*;

public class ChainClient extends JFrame implements Runnable, IChainChannelListener {
	private final static String HOSTNAME = "127.0.0.1";
	private final static int PORT = 107;
	
	private final int GAME_STATE_PENDING = 1;
	private final int GAME_STATE_OPEN = 1;
	private final int GAME_STATE_STARTED = 1;
	
	private final int MAX_X = 6;
	private final int MAX_Y = 5;
	
	IChainChannel myChannel = null;
	SocketChannel mySocketChannel = null;
	Thread myThread = null;
	
	IChainPitchUI myPitch;
	JLabel myState;
	JSplitPane mySplitPane = null;
	ChainLogin myLogin;
	String myCurrentSessionName = new String();
	int myCurrentPlayerID = 0;
	
	int myGameState = GAME_STATE_PENDING;
	
	private int getGameState() {
		return myGameState;
	}
	
	private void setGameState(int value) {
		myGameState = value;
	}

	public ChainClient(String titel) {
		this(titel, HOSTNAME, PORT);
	}

	public ChainClient(String titel, String hostname, int port) {
		super(titel);
		myState = new JLabel("...");
		ActionListener tmpListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
				if (myChannel == null) {
					showState("Es besteht (noch) keine Verbindung...");
				} else {
					myCurrentSessionName = myLogin.getSessionName();
					myChannel.send(ChainMessage.createCreateMsg(myCurrentSessionName, myLogin.getPassword(), MAX_X, MAX_Y));
				}
				
            }
        };
		myLogin = new ChainLogin(tmpListener);
		myLogin.setVisible(true);
		showLogin();
		connect(hostname, port);
		showState("Warte auf Verbindung zu " + hostname + ":" + port);
	}

	public void onConnected(ChainMessage e) {
		showState("Verbindungsaufbau erfolgreich.");
	}

	private void showState(String msg) {
		myState.setText(msg);
		myLogin.setText(msg);
	}
	
	private void showError(String err) {
		JOptionPane.showMessageDialog(this, err, "ChainReaction", JOptionPane.ERROR_MESSAGE);
	}
	
	private void showInformation(String info, int delay) {
		JOptionPane.showMessageDialog(this, info, "ChainReaction", JOptionPane.INFORMATION_MESSAGE );
	}

	public void onReceived(ChainMessage e) {
		String tmpType = e.getType();
		System.out.println("R>" + e.toString());
		if (tmpType.equals(ChainMessage.CHAIN_SOCKET_VERSION)) {
			if (e.getVersion() != 1) {
				showError("Dieser Client untersttzt die Protokoll-Version der Gegenstelle nicht.");
				System.exit(0);
			}
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_CREATE)) {
			setGameState(GAME_STATE_OPEN);
			showState("Das Spiel wurde erstellt. Warte auf Mitspieler ...");
			showPitch();
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_START)) {
			setGameState(GAME_STATE_STARTED);
			myCurrentPlayerID = e.getPlayerID();
			if (myCurrentPlayerID == 0) {
				showState("Das Spiel wurde gestartet. Du fngst an.");
			} else {
				showState("Das Spiel wurde gestartet. Dein Gegner fngt an.");
			}
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_SET)) {
			if (myPitch == null) {
				showState("Interner Fehler (IN:SET - Spielfeld existiert nicht)");
			} else {
				myPitch.addAtom(e.getPlayerID(), e.getX(), e.getY());
				if (myCurrentPlayerID == e.getPlayerID()) {
					showState("Warte auf Gegner ...");
				} else {
					showState("Du bist dran ...");
				}
			}
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_WIN)) {
			if (myCurrentPlayerID == e.getPlayerID()) {
				showInformation("Gratulation. Du hast das Spiel gewonnen.", 5);
			} else {
				showInformation("Du hast leider verloren ...", 5);
			}
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_ERR)) {
			showError(e.getError());
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_CLOSE)) {
			showLogin();
		}
	}

	public void onClosed(ChainMessage e) {
		showState("Die Verbindung wurde unterbrochen.");
	}
	
	private void showPitch() {
		ChainPitchPanel tmpPanel = createPitchComponent();
		mySplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, tmpPanel, myState);
		try { getContentPane().remove(myLogin); } catch(Exception e) {}
		try { getContentPane().add("Center", mySplitPane); } catch(Exception e) {}		
		mySplitPane.setDividerLocation(getContentPane().getHeight()-30);
		
		mySplitPane.setVisible(true);
		myPitch = new ChainPitchUI(MAX_X, MAX_Y, tmpPanel);
		myState.setMinimumSize(new Dimension(myState.getWidth(), 30));
		getContentPane().validate();
		getContentPane().repaint();
	}
	
	private void showLogin() {
		try { getContentPane().remove(mySplitPane); } catch(Exception e) {}
		try { getContentPane().add("Center", myLogin); } catch(Exception e) {}	
		getContentPane().validate();
		getContentPane().repaint();
	}
	
	public ChainPitchPanel createPitchComponent() {
		ChainPitchPanel tmpUI = new ChainPitchPanel();
		tmpUI.setBackground(new Color(44, 44, 44));
		tmpUI.setVisible(true);
		tmpUI.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if (myPitch != null) {
					if (myChannel == null) {
						showState("Es existiert keine Verbindung...");
					} else{
						Point pos = myPitch.translatePos(e.getPoint());
						myChannel.send(ChainMessage.createSetMsg(myCurrentSessionName, pos.x, pos.y));
					}
				}
			}
		});
		return tmpUI;
	}
	
	private boolean connect(String hostname, int port) {
		try {
			mySocketChannel = SocketChannel.open();
			mySocketChannel.configureBlocking(false);		
			mySocketChannel.connect(new InetSocketAddress(hostname, port));
			myThread = new Thread(this);
			myThread.start();
		} catch (Exception e) {
			e.printStackTrace();
			showError("Die Verbindung zum Server konnte nicht hergestellt werden!");
			System.exit(0);
		}
		return true;
	}
	
	public void run() {
		try {
			Selector tmpSel = Selector.open();
			SelectionKey tmpConnectKey = mySocketChannel.register(tmpSel, SelectionKey.OP_CONNECT);
			while (true) {
				try {
					tmpSel.select();
					for (Iterator<SelectionKey> tmpKeyIterator = tmpSel.selectedKeys().iterator(); tmpKeyIterator.hasNext();) {
						SelectionKey tmpKey = tmpKeyIterator.next();
						tmpKeyIterator.remove();
						if (tmpKey.isConnectable()) {
							if(mySocketChannel.isConnectionPending()) mySocketChannel.finishConnect();
							myChannel = new ChainChannel(tmpSel, mySocketChannel, this);
						} else {
							IChainChannel tmpChainChannel = null;
							try {
								tmpChainChannel = (IChainChannel)(tmpKey.attachment());
							} catch (Exception e) {
								e.printStackTrace();
							}
							if (tmpChainChannel != null) {
								tmpChainChannel.process();
							}
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void simulate(String hostname, int port) {
		ChainClient tmpClient = new ChainClient("ChainClient - 1", hostname, port);
		//tmpClient.setSize(720, 650);
		tmpClient.setSize(520, 450);
		tmpClient.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		tmpClient.setLocationRelativeTo(null);
		tmpClient.setVisible(true);
		
		ChainClient tmpClient2 = new ChainClient("ChainClient - 2", hostname, port);
		tmpClient2.setSize(520, 450);
		tmpClient2.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		tmpClient2.setLocationRelativeTo(null);
		tmpClient2.setVisible(true);
		
		tmpClient.setLocation(new Point(120,50));
		tmpClient2.setLocation(new Point(120,500));
	}

	public static void main(String[] args) {
		ChainClient tmpClient = new ChainClient("ChainClient");
		tmpClient.setSize(720, 650);
		tmpClient.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		tmpClient.setLocationRelativeTo(null);
		tmpClient.setVisible(true);
	}


	
	
	
}





