/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.io.*;
import java.net.*;
import java.util.*;
import java.nio.*;
import java.nio.charset.*;
import java.nio.channels.*;

public class ChainServer implements Runnable, IChainChannelListener {
	int myPort = 0;
	ServerSocketChannel myServer = null;
	Thread myThread = null;
	Map<String, IChainSession> mySessionList = null;

	public synchronized boolean removeSession(IChainChannel channel) {
		for (Map.Entry<String, IChainSession> tmpSession : mySessionList.entrySet()) {
			if (tmpSession.getValue().getPlayerID(channel) != -1) {
				if (removeSession(tmpSession.getValue(), "Ein Teilnehmer hat das Spiel verlassen.")) {
					return removeSession(channel);
				}
			}
		}
		return false;
	}

	public synchronized boolean removeSession(IChainSession session, String message) {
		if (session == null) return false;
		mySessionList.remove(session);
		mySessionList.remove(session.getName());
		session.send(ChainMessage.createCloseMsg(session.getName(), message));
		return true;
	}

	public synchronized void onConnected(ChainMessage e) {
		//System.out.println("Connected: " + e.getChannel().getAddress());
		e.getChannel().send(ChainMessage.createVersionMsg("1"));		
	}

	public synchronized void onReceived(ChainMessage e) {
		String tmpType = e.getType();
		System.out.println(e.toString());
		
		IChainPitch tmpPitch = null;
		IChainSession tmpSession = null;
		IChainChannel tmpChannel = e.getChannel();

		if (tmpType.equals(ChainMessage.CHAIN_SOCKET_CREATE)) {
			if (e.getName().length() == 0) {
				tmpChannel.send(ChainMessage.createErrorMsg("Ungltiger Name."));
			} else {
				tmpSession = mySessionList.get(e.getName());
				if (tmpSession == null) {
					tmpSession = new ChainSession(e.getName(), e.getPassword(), e.getSizeX(), e.getSizeY());
					tmpSession.addPlayer(tmpChannel);
					mySessionList.put(e.getName(), tmpSession);
					tmpChannel.send(ChainMessage.createCreateMsg(e.getName(), e.getPassword(), e.getSizeX(), e.getSizeY()));
				} else {
					if (tmpSession.isPasswordValid(e.getPassword()) == false) {
						tmpChannel.send(ChainMessage.createErrorMsg("Das Passwort ist ungltig."));
					} else if (tmpSession.isSizeValid(e.getSizeX(), e.getSizeY()) == false) {
						tmpChannel.send(ChainMessage.createErrorMsg("Das Raster ist ungltig."));
					} else if (tmpSession.isStarted()) {
						tmpChannel.send(ChainMessage.createErrorMsg("Das Spiel luft bereits."));
					} else {
						tmpSession.addPlayer(tmpChannel);
						tmpChannel.send(ChainMessage.createCreateMsg(e.getName(), e.getPassword(), e.getSizeX(), e.getSizeY()));
						tmpSession.start();
					}
				}
			}
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_SET)) {
			tmpSession = mySessionList.get(e.getName());
			if (tmpSession == null) {
				tmpChannel.send(ChainMessage.createErrorMsg("Die Session wurde nicht gefunden."));
			} else {
				if (tmpSession.getPlayerID(tmpChannel) == -1) {
					tmpChannel.send(ChainMessage.createErrorMsg("Ein Spielzug kann nur von Mitgliedern einer Session ausgefhrt werden."));
				} else {
					tmpPitch = tmpSession.getPitch();
					if (tmpPitch == null) {
						tmpChannel.send(ChainMessage.createErrorMsg("Interner Fehler."));
					} else {
						if (tmpSession.isStarted()) {
							if (tmpSession.checkCurrentPlayer(tmpChannel)) {
								if (tmpPitch.addAtom(tmpSession.getPlayerID(tmpChannel), e.getX(), e.getY())) {
									tmpSession.shiftPlayer();
									tmpSession.send(ChainMessage.createSetMsg(e.getName(), e.getX(), e.getY(), tmpSession.getPlayerID(tmpChannel)));
									if (tmpPitch.isDefeated()) {
										tmpSession.send(ChainMessage.createWinMsg(e.getName(), tmpSession.getPlayerID(tmpChannel)));
										removeSession(tmpSession, "Das Spiel wurde beendet.");
									}
								} else {
									tmpChannel.send(ChainMessage.createErrorMsg("Ungltiger Spielzug."));
								}
								//System.out.println(tmpPitch.toString());
							} else {
								tmpChannel.send(ChainMessage.createErrorMsg("Bitte auf die Mitspieler warten ..."));
							}
						} else {
							tmpChannel.send(ChainMessage.createErrorMsg("Das Spiel wurde noch nicht gestartet."));
						}
					}
				}
			}
		}
	}

	public void onClosed(ChainMessage e) {
		removeSession(e.getChannel());	
		System.out.println("Closed: " + e.getChannel().getAddress());
	}

	public ChainServer(int port) {
		myPort = port;
		try {
			mySessionList = new HashMap<String, IChainSession>();
			myServer = ServerSocketChannel.open();
			myServer.socket().bind(new InetSocketAddress(myPort));	
			myServer.configureBlocking(false);
			myThread = new Thread(this);
			System.out.println("Der Server wurde gestartet. Port: " + myPort);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void start() {
		myThread.start();
	}

	public void run() {
		try {
			SocketChannel tmpChannel = null;
			Selector tmpSel = Selector.open();
			SelectionKey tmpAcceptKey = myServer.register(tmpSel, SelectionKey.OP_ACCEPT);
			while (true) {
				try {
					tmpSel.select();
					for (Iterator<SelectionKey> tmpKeyIterator = tmpSel.selectedKeys().iterator(); tmpKeyIterator.hasNext();) {
						SelectionKey tmpKey = tmpKeyIterator.next();
						tmpKeyIterator.remove();
						if (tmpKey == tmpAcceptKey) {
							if (tmpKey.isAcceptable()) {
								tmpChannel = null;
								tmpChannel = myServer.accept();
								tmpChannel.configureBlocking(false);
								new ChainChannel(tmpSel, tmpChannel, this);
							}
						} else {
							IChainChannel tmpChainChannel = null;
							try {
								tmpChainChannel = (IChainChannel)(tmpKey.attachment());
							} catch (Exception e) {
								e.printStackTrace();
							}
							if (tmpChainChannel != null) {
								tmpChainChannel.process();
							}
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		System.out.println("Server: Thread zur Verarbeitung der Verbindungen wurde unerwartet beendet.");
	}

	public static void simulate(int port) {
		new ChainServer(port).start();
	}

	public static void main(String[] args) {
		int port = 0;
		if (args.length > 1) {
			if (args[0].toLowerCase().equals("-p")) {
				try {
					port = Integer.parseInt(args[1]);
				} catch (Exception e) {}
			}
		}
		if (port == 0) port = 107;
		(new ChainServer(port)).start();
	}
	
}





